#include <xsTypes.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

// Functions called FROM here to Fortran:

PROTOCCALLSFSUB19(OXNEISPEC,oxneispec,INT,FLOATV,INT,FLOATV,INTV,INTV,INT,FLOATV,
      FLOAT,FLOATV,INT,FLOATV,INTV,PLOGICAL,STRING,FLOATV,FLOATV,FLOATV,FLOATV)
#define OXNEISPEC(ntemp,temp,nion,ionfrac,ionel,ionstage,ne,ear,z,linesh,nabun, \
           abund,abunel,qcalc,callr,photar,photsum,workphot,earold) \
           CCALLSFSUB19(OXNEISPEC,oxneispec,INT,FLOATV,INT,FLOATV,INTV,INTV,INT, \
           FLOATV,FLOAT,FLOATV,INT,FLOATV,INTV,PLOGICAL,STRING,FLOATV,FLOATV, \
           FLOATV,FLOATV,ntemp,temp,nion,ionfrac,ionel,ionstage,ne,ear,z,linesh,nabun, \
           abund,abunel,qcalc,callr,photar,photsum,workphot,earold)

// Functions called from Fortran TO here:

void oneispec(int ntemp, float* temp, int nion, float* ionfrac, int* ionel, 
        int* ionstage, int ne, float* ear, float z, float* linesh, int nabun,
        float* abund, int* abunel, int* qcalc, char* callr, float* photar);
FCALLSCSUB16(oneispec,ONEISPEC,oneispec,INT,FLOATV,INT,FLOATV,INTV,INTV,INT,
        FLOATV,FLOAT,FLOATV,INT,FLOATV,INTV,PLOGICAL,STRING,FLOATV)




void oneispec(int ntemp, float* temp, int nion, float* ionfrac, int* ionel, 
        int* ionstage, int ne, float* ear, float z, float* linesh, int nabun,
        float* abund, int* abunel, int* qcalc, char* callr, float* photar)
{
   // Routine to return the spectrum from a multi-temperature NEI plasma. 
   // Initial version is simply a wrap-up of Kazik's code and doesn't use 
   // the more sophisticated input arrays. NB The first entry in the abund 
   // array should be for H. This routine grabs memory then calls oxneispec
   // to actually do the work.

   // Arguments
   //    temp         r        i: temperature (keV)
   //    nion         i        i: size of ion arrays (may be > actual no. of ions)
   //    ionfrac      r        i: fraction in ion
   //    ionel        i        i: element for this ion
   //    ionstage     i        i: level for this ion
   //    ne           i        i: number of response energies
   //    ear          r        i: response energies
   //    z            r        i: redshift
   //    linesh       r        i: lineshape parameters
   //    nabun        i        i: number of elements with abundances
   //    abund        r        i: abundances
   //    abunel       i        i: element for this abundance
   //    qcalc        l        i: if true force a recalculation of all arrays
   //    callr        c        i: name of calling routine
   //    photar       r        r: output spectrum
   
   // Translated from oneispec.f 03/09 (CG)
   
   using namespace XSutility;
   
   static bool isFirst = true;
   static int nesave = -1;
   static auto_array_ptr<float> apEarold(0);
   static auto_array_ptr<float> apWorkphot(0);
   
   if (isFirst || ne != nesave)
   {
      float *oldF = apEarold.reset(new float[ne+1]);
      delete [] oldF;
      apEarold[0] = -1.0;
      oldF = apWorkphot.reset(new float[ne*nabun]);
      delete [] oldF;
      
      nesave = ne;
      isFirst = false;
   }
   
   auto_array_ptr<float> apPhotsum(new float[ne*nabun]);
   
   OXNEISPEC(ntemp, temp, nion, ionfrac, ionel, ionstage, ne, ear, z,
      linesh, nabun, abund, abunel, *qcalc, callr, photar, apPhotsum.get(),
      apWorkphot.get(), apEarold.get());
}
